clear;
close all;

% load sample image (scale to 0..1 range)
I = double(imread('cameraman.tif'))/255;
% optionally resize to try on bigger images
%I = imresize(I, 8);
[h, w] = size(I);
N = h*w;

% distort with gaussian noise
sigma = 0.05;
J = I + randn(size(I))*sigma;
f = J(:);

% restoration parameters
lambda = 2.0;

% linear operators: gradient, divergence, laplacian
grad = spmat_gradient2d(h, w);
div = -grad';
lap = div*grad;

% Uncomment to see info about how Matlab solves this eqn system
%spparms('spumoni', 2);

% solve linear system of equations
% 
%    u - f - lambda*lap(u) = 0
%    (I - lambda*lap)*u = f
%
u = (speye(N) - lambda*lap) \ f;

% reshape as image and display
R = reshape(u,h,w);

subplot(1,3,1);
imshow(I);
title("original");

subplot(1,3,2);
imshow(J);
title("noisy");

subplot(1,3,3);
imshow(R);
title("restored");
