function [grad,dx,dy] = spmat_gradient2d(ny, nx)
%spmat_gradient2d Assembles linear operator for gradient
%   Input args are nx, ny, the dimension of the image
%   This implementation is for single channel images.
%   The resulting gradient operates on a column wise stacked image vector.
%   Gradient in x and y direction are stacked vertically, therefore the
%   Output is a (2*ny*nx)x(ny*nx) sparse matrix for the gradient, and
%   (ny*nx)*(ny*nx) sparse matrices for dx and dy.

    % dy:
    % -1 1
    %   -1 1
    %      ...
    %        -1 1
    %          0 0 
    %            -1 1
    %              -1 1
    %                 ...
    %                  -1 1
    %                     0 0
    %                       ...
    dy = spdiags([[-ones(ny - 1, 1); 0], ones(ny, 1)], [0, 1], ny, ny);
    dy = kron(speye(nx), dy);
    
    % dx:
    % -1     ... 1
    %   -1     ... 1
    %     -1     ... 1
    %       -1     ... 1
    %         ...
    %           0
    %             0
    %               0
    %                 ...
    dx = spdiags([[-ones(ny*(nx-1),1); zeros(ny, 1)], ones(nx*ny,1)], ...
                 [0, ny], nx*ny,nx*ny);
    
    grad = [dx; dy];
end